/* 
 * Gain access to the Prefences service.
 */
var prefManager = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefBranch);

/*
 * Called when the preference window loads.  Populates the sites List
 * based on the "extensions.timeshift.sites" preference.
 */
function populateSitesList() {
	var prefString = prefManager.getCharPref("extensions.timeshift.sites");
	if(prefString == "")
		return;

	var sites = prefString.split(".NEXT.");
	var sitesList = document.getElementById("sitesList");

	for (var i=0; i < sites.length; i++) {
		var pieces = sites[i].split(".ITEM.");

		var newItem = document.createElement("treeitem");
		var newRow = document.createElement("treerow");
		newItem.appendChild(newRow);

            var site = document.createElement("treecell");
		site.setAttribute("label", pieces[0]);
		newRow.appendChild(site);

	      var offset = document.createElement("treecell");
		offset.setAttribute("label", pieces[1]);
		newRow.appendChild(offset);

	      var format = document.createElement("treecell");
		format.setAttribute("label", dateFormatText(pieces[2]));
		format.setAttribute("value", pieces[2]);
		newRow.appendChild(format);

		sitesList.appendChild(newItem);
      }
}



/*
 * Save the sites List to the "extensions.timeshift.sites" preference.
 */
function saveSitesList() {
	var sitesList = document.getElementById("sitesList").childNodes;
	var sites = document.getElementById("sites");
	var prefString = "";

	for (var i=0; i < sitesList.length; i++) {
		var columns = sitesList[i].childNodes[0].childNodes;
		var str = columns[0].getAttribute("label") + ".ITEM."
                      + columns[1].getAttribute("label") + ".ITEM."
			    + columns[2].getAttribute("value");
		if(prefString == "")
			prefString = str;
		else
			prefString += ".NEXT." + str;
      }
	sites.value = prefString;
}



/*
 * Populates Site Editor with the given info
 */
function populateEdit(site, offset, format, editIndexStr) {
	var sitesField = document.getElementById("sitesField");
	var offsetField = document.getElementById("offsetField");
	var formatField = document.getElementById("formatField");
	var editIndex = document.getElementById("editIndex");

	editIndex.value = "" + editIndexStr;
	sitesField.value = "" + site;
	offsetField.value = "" + offset;
	formatField.value = "" + format;

	sitesField.focus();
}



/*
 * Edits the currently selected Site
 */
function editSite() {
	var sitesTree = document.getElementById("sitesTree");
	var selectedIndex = sitesTree.currentIndex;

	/* Ignore the button if no Site selected */
	if(selectedIndex == -1)
		return;

	var sitesList = document.getElementById("sitesList");
	var entry = sitesList.childNodes[selectedIndex].childNodes[0].childNodes;

	populateEdit(entry[0].getAttribute("label"),
		       entry[1].getAttribute("label"),
		       entry[2].getAttribute("value"),
			 selectedIndex);
}



/*
 * Deletes the currently selected Site
 */
function deleteSite() {
	var sitesTree = document.getElementById("sitesTree");
	var index = sitesTree.currentIndex;

	if(index != -1) {
		var sitesList = document.getElementById("sitesList");
		var toRemove = sitesList.childNodes.item(index);
		sitesList.removeChild(toRemove);
		saveSitesList();
	}
}


/*
 * Called when a Site has been created/modified
 */
function saveSite() {
	var sitesList = document.getElementById("sitesList");
	var sitesInput = document.getElementById("sitesField");
	var offsetInput = document.getElementById("offsetField");
	var formatInput = document.getElementById("formatField");
	var editIndex = document.getElementById("editIndex");

	if(sitesInput.value == "" || offsetInput.value == "" || formatInput.value == "") {
		alert("You must enter a site, offset and date format!");
		return;
	}

	var newRow = document.createElement("treerow");
	var newItem = document.createElement("treeitem");
	newItem.appendChild(newRow);

	var sites = document.createElement("treecell");
	sites.setAttribute("label", sitesInput.value);
	newRow.appendChild(sites);

	var offset = document.createElement("treecell");
	offset.setAttribute("label", offsetInput.value);
	newRow.appendChild(offset);

	var format = document.createElement("treecell");
	format.setAttribute("label", dateFormatText(formatInput.value));
	format.setAttribute("value", formatInput.value);
	newRow.appendChild(format);

	if(editIndex.value == "") {
		sitesList.appendChild(newItem);
	} else {
		var oldItem = sitesList.childNodes[parseInt(editIndex.value)];
		sitesList.replaceChild(newItem, oldItem);
	}
	saveSitesList();
	populateEdit("","","","");
}


/*
 * Called to Clear the Site Editor.
 */
function cancelSite() {
	populateEdit("","","","");
}


/*
 * Moves the selected item up/down one place
 */
function move(dir) {
	var sitesTree = document.getElementById("sitesTree");
	var index = sitesTree.currentIndex;

	if(index != -1) {
		var sitesList = document.getElementById("sitesList");
		if(dir == "up" && index > 0) {
			var nextIndex = index - 1;
			var top = sitesList.childNodes[nextIndex];
			var bottom = sitesList.childNodes[index];
		} else if(dir == "down" && index < sitesList.childNodes.length - 1) {
			var nextIndex = index + 1;
			var top = sitesList.childNodes[index];
			var bottom = sitesList.childNodes[nextIndex];
		} else {
			return;
		}

		var oA = top.childNodes[0].childNodes[0].getAttribute("label");
		var oB = top.childNodes[0].childNodes[1].getAttribute("label");
		var oCl = top.childNodes[0].childNodes[2].getAttribute("label");
		var oCv = top.childNodes[0].childNodes[2].getAttribute("value");
	
		var iA = bottom.childNodes[0].childNodes[0].getAttribute("label");
		var iB = bottom.childNodes[0].childNodes[1].getAttribute("label");
		var iCl = bottom.childNodes[0].childNodes[2].getAttribute("label");
		var iCv = bottom.childNodes[0].childNodes[2].getAttribute("value");
			
		top.childNodes[0].childNodes[0].setAttribute("label", iA);
		top.childNodes[0].childNodes[1].setAttribute("label", iB);
		top.childNodes[0].childNodes[2].setAttribute("label", iCl);
		top.childNodes[0].childNodes[2].setAttribute("value", iCv);

		bottom.childNodes[0].childNodes[0].setAttribute("label", oA);
		bottom.childNodes[0].childNodes[1].setAttribute("label", oB);
		bottom.childNodes[0].childNodes[2].setAttribute("label", oCl);
		bottom.childNodes[0].childNodes[2].setAttribute("value", oCv);

		sitesTree.currentIndex = nextIndex;
		sitesTree.focus();
		saveSitesList();
	}
}

function dateFormatText(listValue) {
	var stringsBundle = document.getElementById('string-bundle');
	var listGlobal = stringsBundle.getString('formatMenu_Global');
	var listGood = stringsBundle.getString('formatMenu_Good');
	var listBad = stringsBundle.getString('formatMenu_Bad');
	var returnValue = "";
	if(listValue == "0") {
		returnValue = listGlobal;
	} else if(listValue == "1") {
		returnValue = listGood;
	} else if(listValue == "2") {
		returnValue = listBad;
	}
	return returnValue;
}


